@@ -1,3 +1,21 @@
+#
+#     Perl DateTime extension for computing the sunrise/sunset on a given day
+#     Copyright (C) 1999-2013 Ron Hill and Jean Forget
+# 
+#     This program is free software; you can redistribute it and/or modify
+#     it under the terms of the GNU General Public License as published by
+#     the Free Software Foundation; either version 1, or (at your option)
+#     any later version.
+# 
+#     This program is distributed in the hope that it will be useful,
+#     but WITHOUT ANY WARRANTY; without even the implied warranty of
+#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#     GNU General Public License for more details.
+# 
+#     You should have received a copy of the GNU General Public License
+#     along with this program; if not, write to the Free Software Foundation,
+#     Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+#
 package DateTime::Event::Sunrise;
 
 use strict;
@@ -11,7 +29,7 @@ use Params::Validate qw(:all);
 use Set::Infinite qw(inf $inf);
 use vars qw( $VERSION $RADEG $DEGRAD @ISA );
 @ISA     = qw( Exporter );
-$VERSION = '0.0501';
+$VERSION = '0.0502';
 $RADEG   = ( 180 / pi );
 $DEGRAD  = ( pi / 180 );
 my $INV360 = ( 1.0 / 360.0 );
@@ -37,8 +55,6 @@ sub new {
     return bless \%args, $class;
 }
 
-sub sunrise {
-
     #
     #
     # FUNCTIONAL SEQUENCE for sunrise 
@@ -55,6 +71,7 @@ sub sunrise {
     #
     # A new DateTime::Set recurrence object 
     #
+sub sunrise {
 
     my $class = shift;
     my $self  = $class->new(@_);
@@ -69,8 +86,6 @@ sub sunrise {
       } );
 }
 
-sub sunset {
-
     #
     #
     # FUNCTIONAL SEQUENCE for sunset  
@@ -87,6 +102,7 @@ sub sunset {
     #
     # A new DateTime::Set recurrence object
     #
+sub sunset {
 
     my $class = shift;
     my $self  = $class->new(@_);
@@ -101,8 +117,6 @@ sub sunset {
       } );
 }
 
-sub sunset_datetime {
-
     #
     #
     # FUNCTIONAL SEQUENCE for sunrise_sunset 
@@ -122,19 +136,19 @@ sub sunset_datetime {
     #
     #  DateTime object that contains the set time
     #
+sub sunset_datetime {
+
     my $self  = shift;
     my $dt    = shift;
     my $class = ref($dt);
 
-    if ( $class ne 'DateTime' ) {
+    if ( ! $dt->isa('DateTime') ) {
         croak("Dates need to be DateTime objects");
     }
     my ( undef, $tmp_set ) = _sunrise( $self, $dt );
     return $tmp_set;
 }
 
-sub sunrise_datetime {
-
     #
     #
     # FUNCTIONAL SEQUENCE for sunrise_sunset 
@@ -154,19 +168,19 @@ sub sunrise_datetime {
     #
     #  DateTime object that contains the rise times
     #
+sub sunrise_datetime {
+
     my $self  = shift;
     my $dt    = shift;
     my $class = ref($dt);
 
-    if ( $class ne 'DateTime' ) {
+    if ( ! $dt->isa('DateTime') ) {
         croak("Dates need to be DateTime objects");
     }
     my ( $tmp_rise, undef ) = _sunrise( $self, $dt );
     return $tmp_rise;
 }
 
-sub sunrise_sunset_span {
-
     #
     #
     # FUNCTIONAL SEQUENCE for sunrise_sunset 
@@ -186,11 +200,13 @@ sub sunrise_sunset_span {
     #
     #  DateTime Span object that contains the rise/set times
     #
+sub sunrise_sunset_span {
+
     my $self  = shift;
     my $dt    = shift;
     my $class = ref($dt);
 
-    if ( $class ne 'DateTime' ) {
+    if ( ! $dt->isa('DateTime') ) {
         croak("Dates need to be DateTime objects");
     }
     my ( $tmp_rise, $tmp_set ) = _sunrise( $self, $dt );
@@ -201,8 +217,6 @@ sub sunrise_sunset_span {
     );
 }
 
-sub _following_sunrise {
-
     #
     #
     # FUNCTIONAL SEQUENCE for _following_sunrise 
@@ -223,6 +237,7 @@ sub _following_sunrise {
     #
     #  A new DateTime object that contains the rise time
     #
+sub _following_sunrise {
 
     my $self = shift;
     my $dt   = shift;
@@ -238,8 +253,6 @@ sub _following_sunrise {
     return $tmp_rise;
 }
 
-sub _previous_sunrise {
-
     #
     #
     # FUNCTIONAL SEQUENCE for _previous_sunrise 
@@ -258,6 +271,7 @@ sub _previous_sunrise {
     #
     # A new DateTime Object that contains the rise time 
     #
+sub _previous_sunrise {
 
     my $self = shift;
     my $dt   = shift;
@@ -273,8 +287,6 @@ sub _previous_sunrise {
     return $tmp_rise;
 }
 
-sub _following_sunset {
-
     #
     #
     # FUNCTIONAL SEQUENCE for _following_sunset  
@@ -293,6 +305,7 @@ sub _following_sunset {
     #
     #  A DateTime object with set time
     #
+sub _following_sunset {
 
     my $self = shift;
     my $dt   = shift;
@@ -308,8 +321,6 @@ sub _following_sunset {
     return $tmp_set;
 }
 
-sub _previous_sunset {
-
     #
     #
     # FUNCTIONAL SEQUENCE for _previous_sunset 
@@ -328,6 +339,7 @@ sub _previous_sunset {
     #
     # A DateTime object with set time 
     #
+sub _previous_sunset {
 
     my $self = shift;
     my $dt   = shift;
@@ -343,8 +355,6 @@ sub _previous_sunset {
     return $tmp_set;
 }
 
-sub _sunrise {
-
     #
     #
     # FUNCTIONAL SEQUENCE for _sunrise 
@@ -368,6 +378,7 @@ sub _sunrise {
     # 
     # two DateTime objects the date and time for sunrise and sunset
     #
+sub _sunrise {
 
     my $self      = shift;
     my $dt        = shift;
@@ -474,8 +485,6 @@ sub _sunrise {
 
 }
 
-sub _sunrise_sunset {
-
     #
     #
     # FUNCTIONAL SEQUENCE for sunrise_sunset 
@@ -491,21 +500,29 @@ sub _sunrise_sunset {
     #
     #  rise and set times as hours (GMT Time) 
     #
+sub _sunrise_sunset {
 
     my ( $d, $lon, $lat, $altit, $h ) = @_;
 
+    # Compute local sidereal time of this moment
     my $sidtime = revolution( GMST0($d) + 180.0 + $lon );
 
-    my ( $sRA, $sdec ) = sun_RA_dec($d);
-    my $tsouth  = 12.0 - rev180( $sidtime - $sRA ) / $h;
-    my $sradius = 0.2666 / $sRA;
+    # Compute Sun's RA + Decl + distance at this moment
+    my ( $sRA, $sdec, $sr ) = sun_RA_dec($d);
 
+    # Compute time when Sun is at south - in hours UT
+    my $tsouth  = 12.0 - rev180( $sidtime - $sRA ) / $h;
+
+    # Compute the Sun's apparent radius, degrees
+    my $sradius = 0.2666 / $sr;
+
+    # Do correction to upper limb, if necessary
     if ($upper_limb) {
         $altit -= $sradius;
     }
 
     # Compute the diurnal arc that the Sun traverses to reach 
-    # the specified altitude altit: 
+    # the specified altitude altit:
 
     my $cost =
       ( sind($altit) - sind($lat) * sind($sdec) ) /
@@ -532,8 +549,6 @@ sub _sunrise_sunset {
 
 }
 
-sub GMST0 {
-
     #
     #
     # FUNCTIONAL SEQUENCE for GMST0 
@@ -553,6 +568,8 @@ sub GMST0 {
     #
     # Sidtime
     #
+sub GMST0 {
+
     my ($d) = @_;
 
     my $sidtim0 =
@@ -562,8 +579,6 @@ sub GMST0 {
 
 }
 
-sub sunpos {
-
     #
     #
     # FUNCTIONAL SEQUENCE for sunpos
@@ -583,6 +598,8 @@ sub sunpos {
     # ecliptic longitude and distance
     # ie. $True_solar_longitude, $Solar_distance
     #
+sub sunpos {
+
     my ($d) = @_;
 
     #                       Mean anomaly of the Sun 
@@ -623,8 +640,6 @@ sub sunpos {
     return ( $Solar_distance, $True_solar_longitude );
 }
 
-sub sun_RA_dec {
-
     #
     #
     # FUNCTIONAL SEQUENCE for sun_RA_dec 
@@ -639,9 +654,11 @@ sub sun_RA_dec {
     #
     # _RETURN
     #
-    # Sun's Right Ascension (RA) and Declination (dec)
+    # Sun's Right Ascension (RA), Declination (dec) and distance (r)
     # 
     #
+sub sun_RA_dec {
+
     my ($d) = @_;
 
     # Compute Sun's ecliptical coordinates 
@@ -662,12 +679,10 @@ sub sun_RA_dec {
     my $RA  = atan2d( $y, $x );
     my $dec = atan2d( $z, sqrt( $x * $x + $y * $y ) );
 
-    return ( $RA, $dec );
+    return ( $RA, $dec, $r );
 
 }    # sun_RA_dec
 
-sub days_since_2000_Jan_0 {
-
     #
     #
     # FUNCTIONAL SEQUENCE for days_since_2000_Jan_0 
@@ -686,6 +701,7 @@ sub days_since_2000_Jan_0 {
     #
     # day number
     #
+sub days_since_2000_Jan_0 {
 
     my ($dt) = @_;
 
@@ -727,8 +743,6 @@ sub atan2d {
     ( $RADEG * atan2( $_[0], $_[1] ) );
 }
 
-sub revolution {
-
     #
     #
     # FUNCTIONAL SEQUENCE for revolution
@@ -746,13 +760,12 @@ sub revolution {
     #
     # the value of the input is >= 0.0 and < 360.0
     #
+sub revolution {
 
     my $x = $_[0];
     return ( $x - 360.0 * floor( $x * $INV360 ) );
 }
 
-sub rev180 {
-
     #
     #
     # FUNCTIONAL SEQUENCE for rev180
@@ -770,13 +783,13 @@ sub rev180 {
     #
     # angle that was reduced
     #
+sub rev180 {
+
     my ($x) = @_;
 
     return ( $x - 360.0 * floor( $x * $INV360 + 0.5 ) );
 }
 
-sub equal {
-
     #
     #
     # FUNCTIONAL SEQUENCE for equal
@@ -794,13 +807,13 @@ sub equal {
     #
     # True if the numbers are equal 
     #
+sub equal {
+
     my ( $A, $B, $dp ) = @_;
 
     return sprintf( "%.${dp}g", $A ) eq sprintf( "%.${dp}g", $B );
 }
 
-sub convert_hour {
-
     #
     #
     # FUNCTIONAL SEQUENCE for convert_hour 
@@ -819,6 +832,7 @@ sub convert_hour {
     # _RETURN
     #
     # number of seconds
+sub convert_hour {
 
     my ( $hour_rise_ut, $hour_set_ut ) = @_;
     my $seconds_rise = floor( $hour_rise_ut * 60 * 60 );
@@ -827,6 +841,8 @@ sub convert_hour {
     return ( $seconds_rise, $seconds_set );
 }
 
+1962; # Hint: by RZ, better known as BD
+
 =head1 NAME
 
 DateTime::Event::Sunrise - Perl DateTime extension for computing the sunrise/sunset on a given day
@@ -1021,6 +1037,13 @@ Usually 2 iterations are enough, in rare cases 3 or 4 iterations may be needed.
 
 =back
 
+=head3 Notes on polar locations
+
+If the location is beyond either polar circle, and if the date is 
+near either solstice, there can be midnight sun or polar night.
+In this case, there is neither sunrise nor sunset, and
+the module C<carp>s that the sun never rises or never sets.
+
 =head2 next current previous contains as_list iterator
 
 See DateTime::Set.
@@ -1057,7 +1080,7 @@ module.
 
 =over 4
 
-=item Paul Schlyer, Stockholm, Sweden 
+=item Paul Schlyter, Stockholm, Sweden 
 
 for his excellent web page on the subject.
 
@@ -1069,7 +1092,7 @@ for suggestions.
 
 =head1 COPYRIGHT and LICENSE
 
-Here is the copyright information provided by Paul Schlyer:
+Here is the copyright information provided by Paul Schlyter:
 
 Written as DAYLEN.C, 1989-08-16
 
@@ -1107,6 +1130,7 @@ DateTime::Set documentation
 
 DateTime::SpanSet documentation
 
+Paul Schlyter's homepage at http://stjarnhimlen.se/english.html
+
 =cut
-1;
 
